@@ -32,28 +32,25 @@ incremental backup.
 Requires the ovirt-imageio-client package.
 """
 
-from __future__ import print_function
-
-import argparse
-import getpass
 import inspect
-import logging
 import os
 import time
 
 from contextlib import closing
 
-import ovirtsdk4 as sdk
-import ovirtsdk4.types as types
-from helpers import imagetransfer
-
 from ovirt_imageio import client
 
-start_time = time.time()
+import ovirtsdk4 as sdk
+import ovirtsdk4.types as types
+
+from helpers import common
+from helpers import imagetransfer
+from helpers import units
+from helpers.common import progress
 
 
 def main():
-    parser = argparse.ArgumentParser(description="Backup VM disks")
+    parser = common.ArgumentParser(description="Backup VM disks")
     subparsers = parser.add_subparsers(title="commands")
 
     full_parser = subparsers.add_parser(
@@ -62,7 +59,6 @@ def main():
 
     full_parser.set_defaults(command=cmd_full)
 
-    add_common_args(full_parser)
     add_download_args(full_parser)
 
     full_parser.add_argument(
@@ -81,7 +77,6 @@ def main():
 
     incremental_parser.set_defaults(command=cmd_incremental)
 
-    add_common_args(incremental_parser)
     add_download_args(incremental_parser)
 
     incremental_parser.add_argument(
@@ -91,7 +86,8 @@ def main():
     incremental_parser.add_argument(
         "--from-checkpoint-uuid",
         required=True,
-        help="Perform incremental backup since the specified checkpoint UUID.")
+        help="Perform incremental backup since the specified checkpoint "
+             "UUID.")
 
     incremental_parser.add_argument(
         "--disk-uuid",
@@ -105,8 +101,6 @@ def main():
 
     start_parser.set_defaults(command=cmd_start)
 
-    add_common_args(start_parser)
-
     start_parser.add_argument(
         "vm_uuid",
         help="UUID of the VM to backup.")
@@ -119,7 +113,8 @@ def main():
 
     start_parser.add_argument(
         "--from-checkpoint-uuid",
-        help="Perform incremental backup since the specified checkpoint UUID.")
+        help="Perform incremental backup since the specified checkpoint "
+             "UUID.")
 
     download_parser = subparsers.add_parser(
         "download",
@@ -127,7 +122,6 @@ def main():
 
     download_parser.set_defaults(command=cmd_download)
 
-    add_common_args(download_parser)
     add_download_args(download_parser)
 
     download_parser.add_argument(
@@ -142,9 +136,10 @@ def main():
     download_parser.add_argument(
         "--incremental",
         action="store_true",
-        help="Download incremental backup data in qcow2 format. The downloaded disk "
-             "should be rebased on the previous backup to restore the disk contents. "
-             "Can be used only if the backup was started with --from-checkpoint-uuid")
+        help="Download incremental backup data in qcow2 format. The "
+             "downloaded disk should be rebased on the previous backup "
+             "to restore the disk contents. Can be used only if the "
+             "backup was started with --from-checkpoint-uuid.")
 
     stop_parser = subparsers.add_parser(
         "stop",
@@ -152,8 +147,6 @@ def main():
 
     stop_parser.set_defaults(command=cmd_stop)
 
-    add_common_args(stop_parser)
-
     stop_parser.add_argument(
         "vm_uuid",
         help="UUID of the VM for the backup.")
@@ -164,12 +157,7 @@ def main():
 
     args = parser.parse_args()
 
-    logging.basicConfig(
-        level=logging.DEBUG if args.debug else logging.INFO,
-        filename="example.log",
-        format="%(asctime)s %(levelname)-7s (%(threadName)s) [%(name)s] %(message)s"
-    )
-
+    common.configure_logging(args)
     args.command(args)
 
 
@@ -181,7 +169,7 @@ def cmd_full(args):
     """
     progress("Starting full backup for VM %s" % args.vm_uuid)
 
-    connection = connect_engine(args)
+    connection = common.create_connection(args)
     with closing(connection):
         args.from_checkpoint_uuid = None
         backup = start_backup(connection, args)
@@ -199,7 +187,7 @@ def cmd_incremental(args):
     """
     progress("Starting incremental backup for VM %s" % args.vm_uuid)
 
-    connection = connect_engine(args)
+    connection = common.create_connection(args)
     with closing(connection):
         backup = start_backup(connection, args)
         try:
@@ -224,7 +212,7 @@ def cmd_start(args):
     else:
         progress("Starting full backup for VM %r" % args.vm_uuid)
 
-    connection = connect_engine(args)
+    connection = common.create_connection(args)
 
     with closing(connection):
         backup = start_backup(connection, args)
@@ -238,7 +226,7 @@ def cmd_download(args):
     """
     progress("Downloading VM %s disks" % args.vm_uuid)
 
-    connection = connect_engine(args)
+    connection = common.create_connection(args)
     with closing(connection):
         download_backup(
             connection, args.backup_uuid, args, incremental=args.incremental)
@@ -252,7 +240,7 @@ def cmd_stop(args):
     """
     progress("Finalizing backup %s" % args.backup_uuid)
 
-    connection = connect_engine(args)
+    connection = common.create_connection(args)
     with closing(connection):
         stop_backup(connection, args.backup_uuid, args)
 
@@ -261,36 +249,6 @@ def cmd_stop(args):
 
 # Argument parsing
 
-def add_common_args(parser):
-    parser.add_argument(
-        "--engine-url",
-        help="URL to the engine server (e.g. https://engine_fqdn:port)")
-
-    parser.add_argument(
-        "--username",
-        help="Username of engine API")
-
-    parser.add_argument(
-        "--password-file",
-        help="Password of engine API (if a file is not specified, read from standard input)")
-
-    parser.add_argument(
-        "-c", "--cafile",
-        help="Path to oVirt engine certificate for verifying server.")
-
-    parser.add_argument(
-        "--insecure",
-        dest="secure",
-        action="store_false",
-        help=("Do not verify server certificates and host name "
-              "(not recommended)."))
-
-    parser.add_argument(
-        "--debug",
-        action="store_true",
-        help="log debug level messages to example.log")
-
-
 def add_download_args(parser):
     parser.add_argument(
         "--max-workers",
@@ -301,6 +259,15 @@ def add_download_args(parser):
              "You may want to use lower number if you back up many disks "
              "in the same time.")
 
+    parser.add_argument(
+        "--buffer-size",
+        type=units.humansize,
+        default=client.BUFFER_SIZE,
+        help="Buffer size per worker. The default ({}) gives good "
+             "performance with the default number of workers. If you use "
+             "smaller number of workers you may want use larger value."
+             .format(client.BUFFER_SIZE))
+
     parser.add_argument(
         "--backup-dir",
         default="./",
@@ -342,7 +309,9 @@ def start_backup(connection, args):
         try:
             backup = backup_service.get()
         except sdk.NotFoundError:
-            raise RuntimeError("Backup {} failed".format(backup.id))
+            failure_event = get_backup_events(connection, backup.id)[0]
+            raise RuntimeError(
+                "Backup {} failed: {}".format(backup.id, failure_event))
 
     if backup.to_checkpoint_id is not None:
         progress(
@@ -423,6 +392,7 @@ def download_disk(connection, backup_uuid, disk, disk_path, args, incremental=Fa
                 args.cafile,
                 incremental=incremental,
                 secure=args.secure,
+                buffer_size=args.buffer_size,
                 progress=pb,
                 **extra_args)
     finally:
@@ -430,6 +400,8 @@ def download_disk(connection, backup_uuid, disk, disk_path, args, incremental=Fa
         imagetransfer.finalize_transfer(connection, transfer, disk)
 
 
+# General helpers
+
 def get_vm_disks(connection, vm_id):
     system_service = connection.system_service()
     vm_service = system_service.vms_service().vm_service(id=vm_id)
@@ -444,32 +416,12 @@ def get_vm_disks(connection, vm_id):
     return disks
 
 
-# General helpers
+def get_backup_events(connection, search_id):
+    events_service = connection.system_service().events_service()
 
-def get_pass(args):
-    if args.password_file:
-        with open(args.password_file) as f:
-            password = f.read().rstrip('\n')
-    else:
-        password = getpass.getpass()
-
-    return password
-
-
-def connect_engine(args):
-    return sdk.Connection(
-        url=args.engine_url + '/ovirt-engine/api',
-        username=args.username,
-        password=get_pass(args),
-        ca_file=args.cafile,
-        insecure=not args.secure,
-        debug=args.debug,
-        log=logging.getLogger(),
-    )
-
-
-def progress(msg):
-    print("[ %5.1f ] %s" % (time.time() - start_time, msg))
+    # Get the backup events arranged from the most recent event to the oldest
+    return [dict(code=event.code, description=event.description)
+            for event in events_service.list(search=str(search_id))]
 
 
 if __name__ == "__main__":
